package com.yizhi.comment.application.feign;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.comment.application.vo.*;
import com.yizhi.comment.application.vo.domain.NewCommentVo;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@FeignClient(name = "comment", contextId = "ManageCommentClient")
public interface ManageCommentClient {

	@PostMapping("/remote/manage/comment/delete")
	public String delateComment(@RequestBody CommentDelVO vo);
	
	@PostMapping("/remote/manage/comment/save")
	public NewCommentVo saveComment(@RequestBody NewCommentVo comment);
	
	@GetMapping("/remote/manage/comment/list")
	public Page<CommentVo> list(@RequestParam(name = "bizId") Long bizId, @RequestParam("bizType") Integer bizType, @RequestParam(name = "pageNo") Integer pageNo,
								@RequestParam(name = "pageSize") Integer pageSize);
	
	/**
	 * 评论（回复）上架
	 * 
	 * @param id
	 * @param type
	 * @return boolean
	 */
	@GetMapping("/remote/manage/comment/up")
	public Boolean up(@RequestParam("id") Long id, @RequestParam("type") Integer type);
	
	/**
	 * 评论（回复）下架
	 * 
	 * @param id
	 * @param type
	 * @return
	 */
	@GetMapping("/remote/manage/comment/down")
	public Boolean down(@RequestParam("id") Long id, @RequestParam("type") Integer type);
	
	@GetMapping("/remote/manage/comment/list/export")
	public Map<String, Object> export(
			@ApiParam(name = "bizId", value = "业务id") @RequestParam(name = "bizId") Long bizId,
			@ApiParam(name = "bizType", value = "业务类型") @RequestParam("bizType") Integer bizType,
			@ApiParam(name = "bizName", value = "业务名称") @RequestParam(name = "bizName") String bizName);
	
	/**
	 * 学员端 评论列表
	 * 
	 * @param accountId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/remote/manage/comment/api/list")
	public Page<NewApiCommentVo> apiList(@RequestParam("bizId") Long bizId, @RequestParam("bizType") Integer bizType,
										 @RequestParam("accountId") Long accountId, @RequestParam("companyId") Long companyId, @RequestParam("pageNo") Integer pageNo,
										 @RequestParam("pageSize") Integer pageSize);
	
	@GetMapping("/remote/manage/comment/getAllComments")
	public List<NewCommentVo> getAllComments();

	/**
	 * 管理端 评论（回复）下架或者是删除 若是评论则包含子级,否则不包含
	 *
	 * @param id
	 * @param type 1:评论、2-回复
	 * @param operationType 1-下架   2-删除
	 * @return
	 */
	@GetMapping("/remote/manage/comment/down/include/children")
	public Integer downIncludeChildren(@RequestParam("id") Long id,
									   @RequestParam("type") Integer type,
									   @RequestParam("operationType") Integer operationType);

	/**
	 *论坛  管理端--单个帖子--回复列表分页
	 * @param type
	 * @param postsId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */

	@GetMapping("/remote/manage/comment/getPageToManage")
	public Page<ManageCommentVo> getPageToManage(@RequestParam("type") Integer type,
												 @RequestParam("postsId") Long postsId,
												 @RequestParam("pageNo") Integer pageNo,
												 @RequestParam("pageSize") Integer pageSize);
	/**
	 * 论坛--单个帖子的回复下载
	 * @return
	 */
	@GetMapping("/remote/manage/comment/forum/export")
	public String exportFromForum(@RequestBody Map map);

	@GetMapping("/remote/manage/comment/getPersonalDataMap")
	public Map<Long, StatisticsForumAccountVo> getPersonalDataMap(@RequestParam("bizType") Integer bizType);
	
	@GetMapping("/remote/manage/comment/getCommentNum")
	public Integer getCommentNum(@RequestParam(name = "bizId") Long bizId,@RequestParam(name = "bizType") Integer bizType);
}
