package com.yizhi.application.controller.remote;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.NewComment;
import com.yizhi.application.domain.NewReply;
import com.yizhi.comment.application.vo.domain.NewCommentVo;
import com.yizhi.comment.application.vo.domain.NewReplyVo;
import com.yizhi.core.application.cache.CacheNamespace;
import com.yizhi.core.application.cache.RedisCache;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.util.application.json.JsonUtil;
import com.yizhi.util.application.sensitive.SensitivewordFilter;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.comment.application.vo.StatisticsForumAccountVo;
import com.yizhi.application.service.ICommentService;
import com.yizhi.application.service.IReplyService;
import com.yizhi.application.task.CommentListExportAsync;
import com.yizhi.application.task.ForumCommentExportAsync;
import com.yizhi.comment.application.vo.CommentDelVO;
import com.yizhi.comment.application.vo.CommentVo;
import com.yizhi.comment.application.vo.ManageCommentVo;
import com.yizhi.comment.application.vo.NewApiCommentVo;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * <p>
 * 评论 前端控制器
 * </p>
 *
 * @author yizhi123
 * @since 2019-12-30
 */
@RestController(value = "CommentControllerRemote")
@RequestMapping("/remote/manage/comment")
public class CommentController {

	@Autowired
	private ICommentService commentService;

	@Autowired
	private IReplyService replyService;

	@Autowired
	private IdGenerator idGenerator;

	@Autowired
	private AccountClient accountClient;

	@Autowired
	private com.yizhi.application.mapper.CommentMapper CommentMapper;
	
	@Autowired
	private CommentListExportAsync commentListExportAsync;
	@Autowired
	private ForumCommentExportAsync forumCommentExportAsync;
	@Autowired
	private TaskExecutor taskExecutor;

    @Autowired
    private RedisCache redisCache;
    
	@PostMapping("/delete")
	public String delateComment(@RequestBody CommentDelVO vo) {
		RequestContext context = ContextHolder.get();
		Long accountId = context.getAccountId();
		String name = context.getAccountName();
		Date date = new Date();
		if (vo.getType().equals(0)) {
			NewComment comment = commentService.selectById(vo.getId());
			if (comment != null) {
				comment.setState(0);
				comment.setUpdateById(accountId);
				comment.setUpdateByName(name);
				comment.setUpdateTime(date);
				if (commentService.updateById(comment)){
					NewComment newCommentVo = commentService.selectById(comment.getId());
					commentService.updatePostsInfo(newCommentVo.getBizId(), newCommentVo.getBizType(), newCommentVo.getSiteId(), null, false,false);
				}
				return "success";
			}
		} else {
			NewReply reply = replyService.selectById(vo.getId());
			if (reply != null) {
				reply.setState(0);
				reply.setUpdateById(accountId);
				reply.setUpdateByName(name);
				reply.setUpdateTime(date);
				if (replyService.updateById(reply)){
					NewReply newReplyVo = replyService.selectById(reply.getId());
					commentService.updatePostsInfo(newReplyVo.getBizId(), newReplyVo.getBizType(), null, null, false,false);
				}
				return "success";
			}
		}
		return "error";
	}

	@PostMapping("/save")
	public NewComment saveComment(@RequestBody NewComment comment) {
		RequestContext context = ContextHolder.get();
		Date date = new Date();
		comment.setId(idGenerator.generate());
		comment.setCreateById(context.getAccountId());
		comment.setCreateByName(context.getAccountName());
		comment.setCreateTime(date);
		comment.setSiteId(context.getSiteId());
		comment.setOrgId(context.getOrgId());
		if (commentService.insert(comment)) {
			return comment;
		}
		return null;
	}

	@GetMapping("list")
	public Page<CommentVo> list(@RequestParam(name = "bizId") Long bizId, @RequestParam("bizType") Integer bizType,
                                @RequestParam(name = "pageNo") Integer pageNo, @RequestParam(name = "pageSize") Integer pageSize) {
		Page<CommentVo> page = new Page<CommentVo>();
		page = commentService.getCommentPage(bizId, bizType, pageNo, pageSize);
		String fullName;
		String name;
		for (CommentVo vo : page.getRecords()) {
			AccountVO account = accountClient.findById(vo.getAccountId());
			name = account.getName();
			fullName = account.getFullName();
			vo.setCommentator(name);
			vo.setCommentatorName(fullName);
			vo.setReplyName("--");
			vo.setContent(StringEscapeUtils.unescapeJava(vo.getContent()));
			vo.setUserAvatar(account.getHeadPortrait());
		}
		return page;
	}

	/**
	 * 评论（回复）上架
	 * 
	 * @param id
	 * @param type
	 * @return boolean
	 */
	@GetMapping("/up")
	public Boolean up(@RequestParam("id") Long id, @RequestParam("type") Integer type) {
		RequestContext context = ContextHolder.get();
		Long accountId = context.getAccountId();
		String name = context.getAccountName();
		Date date = new Date();
		if (type == 0) {
			NewComment comment = commentService.selectById(id);
			if (comment != null) {
				comment.setState(1);
				comment.setUpdateById(accountId);
				comment.setUpdateByName(name);
				comment.setUpdateTime(date);
				commentService.updateById(comment);
				return true;
			} else {
				return false;
			}
		} else {
			NewReply reply = replyService.selectById(id);
			if (reply != null) {
				reply.setState(1);
				reply.setUpdateById(accountId);
				reply.setUpdateByName(name);
				reply.setUpdateTime(date);
				replyService.updateById(reply);
				return true;
			} else {
				return false;
			}
		}
	}

	/**
	 * 评论（回复）下架
	 * 
	 * @param id
	 * @param type
	 * @return
	 */
	@GetMapping("/down")
	public Boolean down(@RequestParam("id") Long id, @RequestParam("type") Integer type) {
		RequestContext context = ContextHolder.get();
		Long accountId = context.getAccountId();
		String name = context.getAccountName();
		Date date = new Date();
		if (type == 0) {
			NewComment comment = commentService.selectById(id);
			if (comment != null) {
				comment.setState(2);
				comment.setUpdateById(accountId);
				comment.setUpdateByName(name);
				comment.setUpdateTime(date);
				commentService.updateById(comment);
				return true;
			} else {
				return false;
			}
		} else {
			NewReply reply = replyService.selectById(id);
			if (reply != null) {
				reply.setState(2);
				reply.setUpdateById(accountId);
				reply.setUpdateByName(name);
				reply.setUpdateTime(date);
				replyService.updateById(reply);
				return true;
			} else {
				return false;
			}
		}
	}

	@GetMapping("/list/export")
	public Map<String, Object> export(
			@ApiParam(name = "bizId", value = "业务id") @RequestParam(name = "bizId") Long bizId,
			@ApiParam(name = "bizType", value = "业务类型") @RequestParam("bizType") Integer bizType,
			@ApiParam(name = "bizName", value = "业务名称") @RequestParam(name = "bizName") String bizName) {
		RequestContext requestContext = ContextHolder.get();
		Date submitTime = new Date();
		Long taskId = submitTime.getTime();
		String serialNo = "USER-EXPORT-" + taskId;
		String taskName = "导出评论信息-" + taskId;
		String result = "任务编号：" + serialNo + "。任务名称：" + taskName;
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("accountId", requestContext.getAccountId());
		map.put("siteId", requestContext.getSiteId());
		map.put("companyId", requestContext.getCompanyId());
		map.put("serialNo", serialNo);
		map.put("taskId", taskId);
		map.put("submitTime", submitTime);
		map.put("taskName", taskName);
		map.put("bizId", bizId);
		map.put("bizType", bizType);
		map.put("bizName", bizName);
		map.put("result", result);
		commentListExportAsync.execute(map, true);
		return map;
	}

	/**
	 * 学员端 评论列表
	 * 
	 * @param courseId
	 * @param accountId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/api/list")
	public Page<NewApiCommentVo> apiList(@RequestParam("bizId") Long bizId, @RequestParam("bizType") Integer bizType,
										 @RequestParam("accountId") Long accountId, @RequestParam("companyId") Long companyId, @RequestParam("pageNo") Integer pageNo,
										 @RequestParam("pageSize") Integer pageSize) {
		Page<NewApiCommentVo> page = commentService.apiCommentPage(bizId, bizType, accountId, pageNo, pageSize);
		List<NewReply> replies = replyService.list(bizId);
		Map<Long, List<NewReplyVo>> listMap = new HashMap<>();
        List<Long> accountIds = new ArrayList<>();
		for (NewReply reply1 : replies) {
			NewReplyVo replyVo=new NewReplyVo();
			BeanUtils.copyProperties(reply1,replyVo);
			if (listMap.containsKey(reply1.getCommentId())) {
				listMap.get(reply1.getCommentId()).add(replyVo);
			} else {
				List<NewReplyVo> replies1 = new ArrayList<>();
				replies1.add(replyVo);
				listMap.put(reply1.getCommentId(), replies1);
			}
		}

		for (NewApiCommentVo listCommentVo : page.getRecords()) {
			if (listMap.containsKey(listCommentVo.getId())) {
				listCommentVo.setReplies(listMap.get(listCommentVo.getId()));
			}
            //需要获得用户信息的评论人id
            if (!accountIds.contains(listCommentVo.getAccountId())) {
                accountIds.add(listCommentVo.getAccountId());
            }
            List<NewReplyVo> children = listCommentVo.getReplies();
            //需要获得用户信息的回复人id
            if (CollectionUtils.isNotEmpty(children)) {
                for (NewReplyVo commentReplyVO : children) {
                    if (!accountIds.contains(commentReplyVO.getCommenterId())) {
                        accountIds.add(commentReplyVO.getCommenterId());
                    }
                }
            }
		}
        //根据用户ids获取用户信息并放到map中
        List<AccountVO> accountVOList = accountClient.idsGet(accountIds);
        Map<Long, AccountVO> accountVOMap = new HashMap<>(16);
        if (CollectionUtils.isNotEmpty(accountVOList)) {
            for (AccountVO accountVO : accountVOList) {
                accountVOMap.put(accountVO.getId(), accountVO);
            }
        }
        getRecords(page, accountVOMap,companyId);
		return page;
	}
	
	@GetMapping("getAllComments")
	public List<NewCommentVo> getAllComments(){
		List<NewComment> result = new ArrayList<>();
		List<NewComment> list = CommentMapper.getallComments();
		if (CollectionUtils.isNotEmpty(list)) {
			result.addAll(list);
			for(NewComment comment:list) {
				NewReply reply = new NewReply();
				reply.setCommenterId(comment.getId());
				List<NewReply> replies = replyService.selectList(new EntityWrapper<NewReply>(reply));
				if (CollectionUtils.isNotEmpty(replies)) {
					for(NewReply vo:replies) {
						NewComment newComment = new NewComment();
						newComment.setId(vo.getId());
						newComment.setBizId(vo.getBizId());
						newComment.setBizType(vo.getBizType());
						newComment.setCreateById(vo.getCreateById());
						result.add(newComment);
					}
				}
			}
		}
		List<NewCommentVo> list2=new ArrayList<>();
		for (NewComment n:result
			 ) {
			NewCommentVo NV=new NewCommentVo();
			BeanUtils.copyProperties(n,NV);
			list2.add(NV);
		}
		return list2;
	}
	
    private void getRecords(Page<NewApiCommentVo> page, Map<Long, AccountVO> accountVOMap, Long companyId) {
        if (CollectionUtils.isNotEmpty(page.getRecords())) {
            Set<String> sensitiveWordList = new HashSet<>();
            String sensitiveValues = (String) redisCache.hget(CacheNamespace.REDIS_SENSITIVE_WORD_INFO, companyId.toString());
            if (StringUtils.isNotBlank(sensitiveValues)) {
                sensitiveWordList = JsonUtil.json2Ojbect(sensitiveValues, Set.class);
            }
            for (NewApiCommentVo commentVO : page.getRecords()) {
                AccountVO accountVO = accountVOMap.get(commentVO.getAccountId());
                //组装评论人信息
                if (null != accountVO) {
                    commentVO.setCommentator((accountVO.getFullName()==null||accountVO.getFullName()=="")?accountVO.getName():accountVO.getFullName());
                    commentVO.setUserAvatar(accountVO.getHeadPortrait() == null ? "" : accountVO.getHeadPortrait());
                }
                //敏感词过滤评论内容
                if (CollectionUtils.isNotEmpty(sensitiveWordList)) {
                    if (null != commentVO.getContent()) {
                        commentVO.setContent(filterWord(commentVO.getContent(), sensitiveWordList));
                    }
                }
                if (CollectionUtils.isNotEmpty(commentVO.getReplies())) {
                    for (NewReplyVo commentReplyVO : commentVO.getReplies()) {
                        AccountVO commentAccount = accountVOMap.get(commentReplyVO.getCreateById());
                        AccountVO replyAccount = accountVOMap.get(commentReplyVO.getCommenterId());
                        //组装回复人信息
                        if (null != commentAccount) {
                            commentReplyVO.setCreateByName(commentAccount.getFullName()==""||commentAccount.getFullName()==null?commentAccount.getName():commentAccount.getFullName());
                        }
                        //组装被回复人信息
                        if (null != replyAccount) {
                        	commentReplyVO.setCommenterName(replyAccount.getFullName()==""||replyAccount.getFullName()==null?replyAccount.getName():replyAccount.getFullName());
                        }

                        //敏感词过滤回复内容
                        if (null != commentReplyVO.getContent()) {
                            commentReplyVO.setContent(filterWord(commentReplyVO.getContent(), sensitiveWordList));
                        }
                    }
                }

            }
        }
    }
    
    /**
     * 过滤敏感词
     *
     * @param content           评论内容
     * @param sensitiveWordList 已设置的敏感词集合
     * @return
     */
    public String filterWord(String content, Set<String> sensitiveWordList) {
        if (sensitiveWordList != null) {
            if (!StringUtils.isBlank(content)) {
                return SensitivewordFilter.sensitiveFilter(content, sensitiveWordList);
            }
        }
        return content;
    }



	/**
	 * 管理端 评论（回复）下架或者删除 若是评论则包含子级,否则不包含
	 *
	 * @param id
	 * @param type 1:评论、2-回复
	 * @param operationType 1-下架   2-删除
	 * @return
	 */
	@GetMapping("/down/include/children")
	public Integer downIncludeChildren(@RequestParam("id") Long id,
									   @RequestParam("type") Integer type,
									   @RequestParam("operationType") Integer operationType) {

		if (type == 1) {
			if (commentService.down(operationType, id) > 0) {
				NewComment newCommentVo = commentService.selectById(id);
				if (operationType == 2) {
					//删除才需要更新
					commentService.updatePostsInfo(newCommentVo.getBizId(), newCommentVo.getBizType(), newCommentVo.getSiteId(), null, false, false);
				}
				return 1;
			}
			return 0;
		} else {
			if (replyService.down(operationType, id) > 0) {
				NewReply newReplyVo = replyService.selectById(id);
				if (operationType == 2) {
					commentService.updatePostsInfo(newReplyVo.getBizId(), newReplyVo.getBizType(), null, null, false, false);
				}
				return 1;
			}
			return 0;
		}

	}

	/**
	 *论坛  管理端--单个帖子--回复列表分页
	 * @param type
	 * @param postsId
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */

	@GetMapping("/getPageToManage")
	@ApiOperation(value = "管理端--单个帖子--回复列表分页", response = ManageCommentVo.class)
	public Page<ManageCommentVo> getPageToManage(@ApiParam("type业务类型 1：帖子  必传 ") @RequestParam("type") Integer type,
												 @ApiParam("帖子id ") @RequestParam("postsId") Long postsId,
												 @ApiParam("pageNo") @RequestParam("pageNo") Integer pageNo,
												 @ApiParam("pageSize ") @RequestParam("pageSize") Integer pageSize){
		return commentService.getPageToManage(type,postsId,pageNo,pageSize);
	}

	/**
	 * 论坛--单个帖子的回复下载
	 * @return
	 */
	@GetMapping("/forum/export")
	public String exportFromForum(@RequestBody Map map) {
		forumCommentExportAsync.execute(map, true);
		return "ok";
	}


	/**
	 * 获取用户的相关数据Map
	 * @param bizType
	 * @return
	 */
	@GetMapping("/getPersonalDataMap")
	public Map<Long, StatisticsForumAccountVo> getPersonalDataMap(@RequestParam("bizType") Integer bizType) {
		return commentService.getPersonalDataMap(bizType);
	}
	
	@GetMapping("/getCommentNum")
	public Integer getCommentNum(@RequestParam(name = "bizId") Long bizId,@RequestParam(name = "bizType") Integer bizType) {
		NewComment comment = new NewComment();
		comment.setBizId(bizId);
		comment.setBizType(bizType);
		comment.setState(1);
		return commentService.selectCount(new EntityWrapper<NewComment>(comment));
	}
}
