package com.yizhi.application.controller.student;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.domain.NewComment;
import com.yizhi.application.domain.NewReply;
import com.yizhi.application.service.ICommentService;
import com.yizhi.application.service.IReplyService;
import com.yizhi.comment.application.constans.Constant;
import com.yizhi.comment.application.vo.domain.NewCommentVo;
import com.yizhi.comment.application.vo.domain.NewReplyVo;
import com.yizhi.comment.application.feign.student.StudentCommentClient;
import com.yizhi.application.service.IInformRecordService;
import com.yizhi.comment.application.vo.CommentParam;
import com.yizhi.comment.application.vo.NewApiCommentVo;
import com.yizhi.comment.application.vo.PostsCommentVo;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 评论 前端控制器
 * </p>
 *
 * @author yizhi123
 * @since 2019-12-30
 */
@RestController
@RequestMapping("/student/comment")
public class StudentCommentController implements StudentCommentClient {

    @Autowired
    private ICommentService commentService;
    @Autowired
    private IReplyService replyService;
    @Autowired
    private IInformRecordService informRecordService;


    /**
     * 论坛 学员端获取分页的评论和分页的回复
     *
     * @param param
     * @return
     */
    @Override
    @PostMapping("/getPage")
    public Page<NewApiCommentVo> getPage(@RequestBody CommentParam param) {
        return commentService.getPage(param);
    }

    /**
     * 论坛  我收到的回复分页
     *
     * @param param
     * @return
     */
    @Override
    @PostMapping("/myReceivedPage")
    public Page<PostsCommentVo> myReceivedPage(@RequestBody CommentParam param) {
        return commentService.myReceivedPage(param);
    }

    /**
     * 论坛  我收到的回复分页--查看对话
     *
     * @param id 回复的id
     * @return
     */
    @Override
    @GetMapping("/catCommunication")
    public List<PostsCommentVo> catCommunication(@RequestParam("id") Long id) {
        return replyService.catCommunication(id);
    }


    @Override
    @PostMapping("/mySendPage")
    public Page<PostsCommentVo> mySendPage(@RequestBody CommentParam param) {
        return commentService.mySendPage(param);
    }

    /**
     * 论坛  我的管理--回复管理--回复列表分页
     *
     * @param type
     * @param postsId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @Override
    @GetMapping("/getPageByPostsId")
    @ApiOperation(value = "我的管理--回复管理--回复列表", response = PostsCommentVo.class)
    public Page<PostsCommentVo> getPageByPostsId(@ApiParam("type必传 ") @RequestParam("type") Integer type,
                                                 @ApiParam("帖子id ") @RequestParam("postsId") Long postsId,
                                                 @ApiParam("pageNo") @RequestParam("pageNo") Integer pageNo,
                                                 @ApiParam("pageSize ") @RequestParam("pageSize") Integer pageSize) {
        return commentService.getPageByPostsId(type, postsId, pageNo, pageSize);
    }

    /**
     * 论坛 评论（回复）--删除（下架） 包含子级回复
     *
     * @param id
     * @param type 1:评论、2-回复
     *        operationType 1-下架   2-删除
     *        isInform  是否是举报
     * @return
     */
    @Override
    @GetMapping("/down")
    public Integer down(@RequestParam("id") Long id, @RequestParam("type") Integer type, @RequestParam("isInform") Boolean isInform) {

        if (type == 1) {
            if (commentService.down(1, id) > 0) {
                if (isInform) {
                    informRecordService.cancelByCommentId(Constant.TASK_TYPE_POSTS, null, type, id);
                }
                NewComment newCommentVo = commentService.selectById(id);
                commentService.updatePostsInfo(newCommentVo.getBizId(), newCommentVo.getBizType(), newCommentVo.getSiteId(), null, false, false);
                return 1;
            }
            return 0;
        } else {
            if (replyService.down(1, id) > 0) {
                if (isInform) {
                    informRecordService.cancelByCommentId(Constant.TASK_TYPE_POSTS, null, type, id);
                }
                NewReply newReplyVo = replyService.selectById(id);
                commentService.updatePostsInfo(newReplyVo.getBizId(), newReplyVo.getBizType(), null, null, false, false);
                return 1;
            }
            return 0;
        }
    }

    @Override
    @PostMapping("/save")
    public Integer studentSave(@RequestBody NewCommentVo comment) {
        return commentService.save(comment);
    }
    /**
     *  论坛我的回复--获取我的回复（收到的回复未查看的数量）
     * @param taskType
     * @param taskIds
     * @return
     */
    @Override
    @GetMapping("/myNum")
    public Integer myNum(@RequestParam("taskType") Integer taskType,
                         @RequestParam(value = "taskId",required = false) List<Long> taskIds) {
        return commentService.myNum(taskType,taskIds);
    }

    @Override
    @ApiOperation(value = "获取单个评论详情，包含回复", response = PostsCommentVo.class)
    @GetMapping("/getCommentDetails")
    public NewApiCommentVo getCommentDetails(@ApiParam("taskType必传 ") @RequestParam("taskType") Integer taskType,
                                                  @ApiParam("业务id ") @RequestParam("taskId") Long taskId,
                                                  @ApiParam("评论id ") @RequestParam("commentId") Long commentId,
                                                  @ApiParam("作者id ") @RequestParam(value = "author",required = false) Long author,
                                                  @ApiParam("回复排序方式 ") @RequestParam("orderType") Integer orderType,
                                                  @ApiParam("回复的pageNo") @RequestParam("pageNo") Integer pageNo,
                                                  @ApiParam("回复的pageSize ") @RequestParam("pageSize") Integer pageSize) {
        return commentService.getCommentDetails(taskType, taskId,commentId,author,orderType, pageNo, pageSize);
    }

}

